#| -*-Scheme-*-

Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015,
    2016, 2017, 2018, 2019 Matthew Birkholz

This file is part of a planetarium plugin for MIT/GNU Scheme Pucked.

This plugin is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This plugin is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this plugin; if not, write to the Free Software Foundation,
Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

|#

(declare (no-type-checks) (no-range-checks))

(define (3d-transform! point transform projection)
  ;; point X transform => projection  The projection may be eq point.
  (define-integrable (t i j)
    (flo:vector-ref transform (fix:+ (fix:* i 3) j)))
  (let ((p0 (flo:vector-ref point 0))
	(p1 (flo:vector-ref point 1))
	(p2 (flo:vector-ref point 2)))
    (define-integrable (f i)
      (flo:+        (flo:* p0 (t i 0))
	     (flo:+ (flo:* p1 (t i 1))
		    (flo:* p2 (t i 2)))))
    (flo:vector-set! projection 0 (f 0))
    (flo:vector-set! projection 1 (f 1))
    (flo:vector-set! projection 2 (f 2))))

(define (3d-multiply! A B C)
  ;; A X B => C  C may be eq A or B.
  (define-integrable (ref m row col)
    (flo:vector-ref m (fix:+ (fix:* (fix:-1+ row) 3) (fix:-1+ col))))
  (define-integrable (f+ a b c)
    (flo:+ a (flo:+ b c)))
  (define-integrable (f* a b) (flo:* a b))
  (let ((B11 (ref B 1 1))  (A11 (ref A 1 1))
	(B12 (ref B 1 2))  (A12 (ref A 1 2))
	(B13 (ref B 1 3))  (A13 (ref A 1 3))
	(B21 (ref B 2 1))  (A21 (ref A 2 1))
	(B22 (ref B 2 2))  (A22 (ref A 2 2))
	(B23 (ref B 2 3))  (A23 (ref A 2 3))
	(B31 (ref B 3 1))  (A31 (ref A 3 1))
	(B32 (ref B 3 2))  (A32 (ref A 3 2))
	(B33 (ref B 3 3))  (A33 (ref A 3 3)))
  (flo:vector-set! C 0 (f+ (f* A11 B11) (f* A12 B21) (f* A13 B31)))
  (flo:vector-set! C 1 (f+ (f* A11 B12) (f* A12 B22) (f* A13 B32)))
  (flo:vector-set! C 2 (f+ (f* A11 B13) (f* A12 B23) (f* A13 B33)))
  (flo:vector-set! C 3 (f+ (f* A21 B11) (f* A22 B21) (f* A23 B31)))
  (flo:vector-set! C 4 (f+ (f* A21 B12) (f* A22 B22) (f* A23 B32)))
  (flo:vector-set! C 5 (f+ (f* A21 B13) (f* A22 B23) (f* A23 B33)))
  (flo:vector-set! C 6 (f+ (f* A31 B11) (f* A32 B21) (f* A33 B31)))
  (flo:vector-set! C 7 (f+ (f* A31 B12) (f* A32 B22) (f* A33 B32)))
  (flo:vector-set! C 8 (f+ (f* A31 B13) (f* A32 B23) (f* A33 B33)))))

(define (make-3d-identity-matrix)
  (flo:3d-matrix 1. 0. 0.
		 0. 1. 0.
		 0. 0. 1.))

(define (make-x-rotation-matrix radians)
  (let ((cos (flo:cos radians))
	(sin (flo:sin radians)))
    (let ((-sin (flo:negate sin)))
      (flo:3d-matrix 1.  0.   0.
		     0. cos -sin
		     0. sin  cos))))

(define (make-y-rotation-matrix radians)
  (let ((cos (flo:cos radians))
	(sin (flo:sin radians)))
    (let ((-sin (flo:negate sin)))
      (flo:3d-matrix cos  0. sin
		     0.  1.  0.
		     -sin  0. cos))))

(define (make-z-rotation-matrix radians)
  (let ((cos (flo:cos radians))
	(sin (flo:sin radians)))
    (let ((-sin (flo:negate sin)))
      (flo:3d-matrix cos -sin  0.
		     sin  cos  0.
		     0.   0.  1.))))

(define-integrable (flo:3d-matrix m11 m12 m13 m21 m22 m23 m31 m32 m33)
  (let ((v (flo:vector-cons 9)))
    (flo:vector-set! v 0 m11)
    (flo:vector-set! v 1 m12)
    (flo:vector-set! v 2 m13)
    (flo:vector-set! v 3 m21)
    (flo:vector-set! v 4 m22)
    (flo:vector-set! v 5 m23)
    (flo:vector-set! v 6 m31)
    (flo:vector-set! v 7 m32)
    (flo:vector-set! v 8 m33)
    v))