#!/bin/sh
# -*-Scheme-*-
#
# Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015,
#     2016, 2017, 2018, 2019 Matthew Birkholz
# 
# This file is part of a planetarium plugin for MIT/GNU Scheme Pucked.
# 
# This plugin is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This plugin is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this plugin; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301, USA.

# Test the Planetarium option.

set -ex
: ${MIT_SCHEME_EXE=mit-scheme}
${MIT_SCHEME_EXE} --prepend-library . <<\EOF
(begin
  (set! *initial-options-file* (merge-pathnames "mit-optiondb.scm"))

  (load-option 'planetarium)

  (if (not (gtk-initialized?))
      (warn "Could not test the planetarium.")
      (begin
	(make-tellurion)

	(let ((env (->environment '(planetarium))))
	  ((access test-julian-day env))
	  ((access test-greenwich-mean-sidereal-time env))
	  ((access test-solar-latitude/longitude env))
	  ((access test-angular-separation env)))

	(let ((env (->environment '(planetarium terrain))))
	  (if (not env)
	      (warn "Could not test terrain viewer.")
	      (begin
		((access test-normals env))
		((access write-function-terrain env)
		 "sample-terrain.txt"
		 -3 3 50
		 -3 3 50
		 (lambda (x y)
		   (let ((d^2+1 (+ (* x x) (* y y) 1)))
		     (* 1000 (/ (sin d^2+1) d^2+1)))))
		(let ((view (make-terrain 'filename "sample-terrain.txt"
					  'rows 50 'columns 50
					  'latitude 0. 'longitude 0.
					  'step .001)))
		  (view 'position -.02 -.02 4000.)
		  (view 'heading 45.)
		  (view 'tilt -40.)))))

	(let ((env (->environment '(gtk gtk-widget))))
	  (let loop ()
	    (sleep-current-thread 1000)
	    (if (not (null? (access toplevel-windows env)))
		(loop))))

	;;(gc-flip)
	;;(sleep-current-thread 100) ;Ensure GC deamons run? Hope they don't?
	(let ((ffi (->environment '(runtime ffi))))
	  (if (not (zero? (car ((access registered-callback-count ffi)))))
	      (error "The planetarium did not clean up its callbacks"))
	  (if (not (null? (access malloced-aliens ffi)))
	      (error "The planetarium did not free allocated memory:"
		     (map (lambda (elt)
			    (let ((alien (weak-car elt)))
			      (if (eq? 'uchar (alien/ctype alien))
				  (c-peek-cstring alien)
				  alien)))
			  (access malloced-aliens ffi)))))
	))
  )
EOF
