#| -*-Scheme-*-

Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015,
    2016, 2017, 2018, 2019 Matthew Birkholz

This file is part of a planetarium plugin for MIT/GNU Scheme Pucked.

This plugin is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This plugin is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this plugin; if not, write to the Free Software Foundation,
Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

|#

;;;; Essential R3RS syntax and procedures.
;;; package: (r3rs essentials)

(define-syntax essential-if
  (syntax-rules ()
    ((_ pred cons alt)
     (if pred cons alt))))

(define-syntax essential-let
  (syntax-rules ()
    ((_ (bindings ...) body ...)
     (let (bindings ...) body ...))))

(define-integrable (essential-* a b)
  (* a b))

(define-integrable (essential-+ a b)
  (+ a b))

(define-integrable (essential-- a b)
  (- a b))

(define-integrable (essential-/ a b)
  (/ a b))

(define (essential-append a b)
  (append a b))

(define (essential-apply proc args)
  (apply proc args))

(define (essential-max a b)
  (max a b))

(define (essential-min a b)
  (min a b))

(define (essential-string-append a b)
  (string-append a b))

(define (r3rs-number->string number format)
  (cond ((and (equal? format '(int))
	      (integer? number))
	 (number->string number))
	((and (eq? (car format) 'fix)
	      (integer? (cadr format))
	      (null? (cddr format)))
	 (let* ((whole (truncate->exact number))
		(fract (round->exact
			(* (expt 10 (cadr format)) (abs (- number whole))))))
	   (string-append (number->string whole)
			  "."
			  (string-pad-left (number->string fract)
					   (cadr format) #\0))))
	(else
	 (warn "Unimplemented: r3rs-number->string:" number format)
	 (number->string number))))

(define (r3rs-string->number string exactness radix)
  (let ((n (string->number string (case radix
				    ((B) 2)
				    ((O) 8)
				    ((D) 10)
				    ((X) 16)
				    (else (error "Not a radix:" radix))))))
    (if (not n) (error "Not a number:" string))
    (case exactness
      ((E) (inexact->exact n))
      ((I) (exact->inexact n))
      (else (error "Not an exactness:" exactness)))))

(define-syntax r3rs-atan
  (syntax-rules ()
    ((_ t) (atan t))
    ((_ y x) (atan2 y x))))