#| -*-Scheme-*-

Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015,
    2016, 2017, 2018, 2019 Matthew Birkholz

This file is part of a planetarium plugin for MIT/GNU Scheme Pucked.

This plugin is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This plugin is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this plugin; if not, write to the Free Software Foundation,
Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

|#

;;;; Generate snapshots (PNG files).

;; Expect warning about DISPLAY not set.
(load-option 'cairo)

(with-working-directory-pathname
    (directory-pathname (current-load-pathname))
  (lambda ()
    (load-package-set "mit")))

(let ((planet (->environment '(planetarium)))
      (graphics (->environment '(planetarium cairo-graphics))))
  (for-each (lambda (name) (environment-link-name planet graphics name))
	    '(draw-segment draw-circle draw-text
	      fill-polygon-available? fill-polygon
	      clear-graphics flush-graphics)))

(let ((here (the-environment))
      (planet (->environment '(planetarium))))
  (for-each (lambda (name) (environment-link-name here planet name))
	    '(make-latitude/longitude draw-tellurion)))

(let ((time (get-universal-time))
      (lat/long (make-latitude/longitude 64.844 -147.723)) ;Fairbanks
      (device (make-graphics-device 'cairo 400 400)))
  (graphics-set-coordinate-limits device -1.1 -1.1 1.1 1.1)

  (call-with-append-file "tellurion.log"
    (lambda (out)
      (with-notification-output-port out
        (lambda ()
	  (with-gc-notification! #t
	   (lambda ()
	     (show-time
	      (lambda ()
		(draw-tellurion device time lat/long)))))
	  (write-char #\tab out)
	  (write-string (universal-time->local-time-string time) out)
	  (newline out)))))

  (cairo-graphics/write-to-png device "tellurion.png")
  (cairo-graphics/destroy device))