#| -*-Scheme-*-

Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015,
    2016, 2017, 2018, 2019 Matthew Birkholz

This file is part of a planetarium plugin for MIT/GNU Scheme Pucked.

This plugin is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This plugin is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this plugin; if not, write to the Free Software Foundation,
Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

|#

;;;; System specific code for MIT Scheme with X graphics.

(define (make-suitable-graphics-device)
  (let ((device (make-graphics-device 'x)))
    (graphics-set-coordinate-limits device -1.1 -1.1 1.1 1.1)
    (x-graphics/set-background-color device "white")
    (x-graphics/set-foreground-color device "black")
    device))

(define-integrable (x p) (flo:vector-ref p 0))
(define-integrable (y p) (flo:vector-ref p 1))

(define (draw-segment device start end color)
  (x-graphics/set-foreground-color device color)
  (x-graphics/draw-line device
			(x start) (y start)
			(x end) (y end)))

(define (draw-circle device center radius color)
  (x-graphics/set-foreground-color device color)
  (x-graphics/draw-circle device (x center) (y center) radius))

(define (draw-text device position string color)
  (x-graphics/set-foreground-color device color)
  (x-graphics/draw-text device
			(x position) (y position)
			string))

(define (fill-polygon-available?) #f)

(define (fill-polygon device points color)
  (error "Unimplemented: fill-polygon:" device points color))

#;(define (points->vector points)
  (let ((v (make-vector (* 2 (length points)))))
    (let loop ((i 0) (points points))
      (if (pair? points)
	  (let ((point (car points)))
	    (vector-set! v i (x point))
	    (vector-set! v (fix:1+ i) (y point))
	    (loop (fix:+ i 2) (cdr points)))))
    v))

#;(define (fill-polygon device points color)
  (x-graphics/set-foreground-color device color)
  (x-graphics/fill-polygon device (points->vector points)))

(define (clear-graphics device)
  (x-graphics/clear device))

(define (flush-graphics device)
  (x-graphics/flush device))