#| -*-Scheme-*-

Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015,
    2016, 2017, 2018, 2019 Matthew Birkholz

This file is part of a planetarium plugin for MIT/GNU Scheme Pucked.

This plugin is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This plugin is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this plugin; if not, write to the Free Software Foundation,
Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

|#

;;; Formula numbers and unattributed quotes are from Meeus 2009,
;;; specifically the second English edition (1998) "with corrections
;;; as of August 10, 2009" of _Astronomical_Algorithms_ by Jean Meeus.

(define (universal-time->julian-day time)
  ;; The Julian Ephemeris Day (JDE) corresponding to TIME.
  (let ((decoded (universal-time->global-decoded-time time)))
    (julian-day (decoded-time/year decoded)
		(decoded-time/month decoded)
		(+ (decoded-time/day decoded)
		   (/ (+ (decoded-time/hour decoded)
			 (/ (+ (decoded-time/minute decoded)
			       (/ (decoded-time/second decoded)
				  60))
			    60))
		      24)))))

(define (julian-day year month day)
  (let ((D day)
	(Y (if (< month 3) (- year 1) year))
	(M (if (< month 3) (+ month 12) month)))
    (define-integrable int truncate)
    (let* ((A (int (/ Y 100)))
	   (B (if (or (< year 1582)
		      (and (= year 1582) (< month 10))
		      (and (= year 1582) (= month 10) (< day 15)))
		  0
		  (+ 2 (- A) (int (/ A 4))))))
      (->flonum
       (+ (int (* 365.25 (+ Y 4716)))
	  (int (* 30.6001 (+ M 1)))
	  D
	  B
	  -1524.5)))))

(define (test-julian-day)
  (for-each
    (lambda (item)
      (let ((y (car item))
	    (m (cadr item))
	    (d (caddr item))
	    (jd (cadddr item)))
	(let ((computed (->flonum (julian-day y m d))))
	  (if (not (flo:~= computed jd 0.001))
	      (warn "test-julian-day failed:" y m d jd computed)))))
    '((1957 10 4.81 2436116.31)
      (333 1 27.5 1842713.0)

      (2000  1  1.5 2451545.0)
      (1999  1  1.0 2451179.5)
      (1987  1 27.0 2446822.5)
      (1987  6 19.5 2446966.0)
      (1988  1 27.0 2447187.5)
      (1988  6 19.5 2447332.0)
      (1900  1  1.0 2415020.5)
      (1600  1  1.0 2305447.5)
      (1600 12 31.0 2305812.5)
      ( 837  4 10.3 2026871.8)
      (-123 12 31.0 1676496.5)
      (-122  1  1.0 1676497.5)
      (-1000  7 12.5 1356001.0)
      (-1000  2 29.0 1355866.5)
      (-1001  8 17.9 1355671.4)
      (-4712  1  1.5 0.0))))